<?php
//
// New REST API utils and classes for Nagios XI 5
// Copyright (c) 2015-2018 Nagios Enterprises, LLC. All rights reserved.
//

// Bootstrap CCM functionality
include_once(dirname(__FILE__) . '/../../includes/components/ccm/bootstrap.php');

//
// ACTUAL API SECTION
//

class API extends NagiosAPI
{
    protected $user;

    public function __construct($request, $origin)
    {
        parent::__construct($request);

        // List of nodes that do not need API key
        $no_api_key = array('authenticate', 'license');

        // Do not need an API key to use the authenticate method
        $nodes = explode('/', $request, 2);
        if (in_array($nodes[0], $no_api_key)) {
            return;
        }

        $apikey = new APIKey();

        // If we used a fusekey, we need to swap that into apikey, and check for specified user data [name/id]
        $userinfo = '';
        if (array_key_exists('fusekey', $this->request)) {
            $this->request['apikey'] = $this->request['fusekey'];
            if (array_key_exists('user', $this->request)) {
                $userinfo = $this->request['user'];
            }
        }
        
        if (!array_key_exists('apikey', $this->request) && !array_key_exists('fusekey', $this->request)) {
            throw new Exception(_('No API Key provided'));
        } else if (!$apikey->verify_key($this->request, $this->method, $origin, $reason, $userinfo)) {
            throw new Exception($reason);
        }
    }

    protected function convert_to_output_type($data)
    {
        $type = grab_array_var($this->request, 'outputtype', 'json');
        if ($type == 'xml') {
            return $data;
        }
        $data = simplexml_load_string($data);
        return json_encode($data);
    }

    /**
     * Used for the api/v1/authenticate token generation
     */
    protected function authenticate($verb, $args)
    {
        switch ($this->method)
        {
            case 'POST':
                return $this->authenticate_user();
                break;

            default:
                return array('error' => _('You can only use POST with authenticate.'));
                break;
        }
    }

    /**
     * Used as an endpoint for the V2 license API to poke XI and make it do "things"
     */
    protected function license($verb, $args)
    {
        if (is_v2_license()) {
            switch ($this->method)
            {
                case 'POST':
                    switch ($verb) {

                        case 'checkin':
                            
                            // Rate limit the checkin function to once every 5 seconds
                            $lcc = get_option('last_api_v2_license_checkin', 0);
                            if ($lcc > time() - 5) {
                                return array('error' => _('Slow down. You must wait before requesting another check-in.'));
                            }

                            // Do the actual checkin
                            $ret = pull_v2_license_info();
                            if ($ret) {
                                set_option('last_api_v2_license_checkin', time());
                                return array('success' => _('Successfully ran v2 license check-in.'));
                            } else {
                                return array('error' => _('Could not get updated v2 license information from v2 API.'));
                            }

                            break;

                    }
                    break;
            }
        }
        return array('error' => _('Unknown API endpoint.'));
    }

    protected function objects($verb, $args)
    {
        switch ($this->method)
        {
            case 'GET':
                switch ($verb)
                {
                    case 'hoststatus':
                        $data = get_host_status_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'servicestatus':
                        $data = get_service_status_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'logentries':
                        $data = get_logentries_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'statehistory':
                        $data = get_statehistory_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'comment':
                        $data = get_comments_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'downtime':
                        $data = get_scheduled_downtime_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'contact':
                        $data = get_contact_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'host':
                        $data = get_host_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'service':
                        $data = get_service_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'hostgroup':
                        $data = get_hostgroup_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'servicegroup':
                        $data = get_servicegroup_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'contactgroup':
                        $data = get_contactgroup_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'hostgroupmembers':
                        $data = get_hostgroup_member_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'servicegroupmembers':
                        $data = get_servicegroup_member_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'contactgroupmembers':
                        $data = get_contactgroup_member_objects_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'rrdexport':
                        $data = get_rrd_json_output($this->request);
                        return $data;
                        break;

                    case 'cpexport':
                        require_once('../../includes/components/capacityplanning/cp-common.inc.php');
                        $data = get_cp_array_data();
                        return $data;
                        break;

                    case 'hostavailability':
                        $data = array();
                        $data['hostavailability'] = get_parsed_nagioscore_csv_availability_xml_output('host', $this->request, true);
                        return $data;
                        break;

                    case 'serviceavailability':
                        $data = array();
                        $data['serviceavailability'] = get_parsed_nagioscore_csv_availability_xml_output('service', $this->request, true);
                        return $data;
                        break;

                    case 'sla':
                        // Verify enterprise license
                        if (!enterprise_features_enabled()) {
                            return array("error" => _('Enterprise features are not enabled. Please check your license settings.'));
                        }
                        $data = get_sla_data();
                        return $data;
                        break;

                    case 'bpi':
                        $type = grab_array_var($this->request, 'outputtype', 'json');
                        $data = get_bpi_data($type);
                        return $data;
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'POST':
                return array('error' => _('Unknown API endpoint.'));
                break;

            case 'PUT':
                return array('error' => _('Unknown API endpoint.'));
                break;

            case 'DELETE':
                return array('error' => _('Unknown API endpoint.'));
                break;
        }
    }

    protected function config($verb, $args)
    {
        if (is_admin() == false) { return array('error' => _('Authenticiation failed.')); }

        switch ($this->method)
        {
            case 'GET':
                return array('info' => _('Not implemented yet. Use a POST request to add objects or a DELETE request to remove them.'));
                break;

            case 'POST':
                switch ($verb)
                {
                    case 'import':
                        return $this->raw_cfg_import();
                        break;

                    case 'host':
                        return $this->create_cfg_host();
                        break;

                    case 'service':
                        return $this->create_cfg_service();
                        break;

                    case 'hostgroup':
                        return $this->create_cfg_hostgroup();
                        break;

                    case 'servicegroup':
                        return $this->create_cfg_servicegroup();
                        break;

                    case 'command':
                        return $this->create_cfg_command();
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'PUT':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'DELETE':
                switch ($verb)
                {
                    case 'host':
                        return $this->remove_cfg_host();
                        break;

                    case 'service':
                        return $this->remove_cfg_service();
                        break;

                    case 'command':
                        return $this->remove_cfg_command();
                        break;
                }
                break;
        }

    }

    protected function system($verb, $args)
    {
        if (is_admin() == false) { return array('error' => _('Authenticiation failed.')); }

        switch ($this->method)
        {
            case 'GET':
                switch ($verb)
                {
                    case 'user':
                        return $this->get_users(@$args[0]);
                        break;

                    case 'status':
                        return $this->get_system_status(@$args[0]);
                        break;

                    case 'statusdetail':
                        $data = get_sysstat_data_xml_output($this->request);
                        return $this->convert_to_output_type($data);
                        break;

                    case 'applyconfig':
                        return $this->apply_config();
                        break;

                    case 'importconfig':
                        return $this->import_config();
                        break;

                    case 'info':
                        return $this->get_system_info();
                        break;

                    case 'authserver':
                        return $this->get_auth_servers();
                        break;

                    case 'command':
                        return $this->get_commands(@$args[0]);
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'POST':
                switch ($verb)
                {
                    case 'user':
                        return $this->add_user();
                        break;

                    case 'applyconfig':
                        return $this->apply_config();
                        break;

                    case 'importconfig':
                        return $this->import_config();
                        break;

                    case 'addfusehost':
                        return $this->add_fuse_host();
                        break;

                    case 'authserver':
                        return $this->add_auth_server();
                        break;

                    case 'scheduleddowntime':
                        return $this->add_scheduled_downtime();
                        break;

                    case 'corecommand':
                        return $this->core_command();
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'PUT':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'DELETE':
                switch ($verb)
                {
                    case 'user':
                        return $this->delete_user($args[0]);
                        break;

                    case 'removefusehost':
                        return $this->remove_fuse_host($args[0]);
                        break;

                    case 'authserver':
                        return $this->remove_auth_server($args[0]);
                        break;

                    case 'scheduleddowntime':
                        return $this->remove_scheduled_downtime($args[0]);
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;
        }
    }

    /**
     * Individual user info/account settings, status, etc
     * TODO: Implement actions for users
     */
    protected function user($verb, $args)
    {
        switch ($this->method)
        {
            case 'GET':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'POST':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'PUT':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'DELETE':
                return array('info' => _('This section has not yet been implemented.'));
                break;
        }
    }

    protected function apply_config()
    {
        $cmd_id = submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
        if (empty($cmd_id) || $cmd_id == -1) {
            return array('error' => _('Apply config command failed to be sent to the command subsystem.'));
        }
        return array('success' => _('Apply config command has been sent to the backend.'), 'command_id' => $cmd_id);
    }

    protected function import_config()
    {
        $cmd_id = submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
        if (empty($cmd_id) || $cmd_id == -1) {
            return array('error' => _('Import config command failed to be sent to the command subsystem.'));
        }
        return array('success' => _('Import configs command has been sent to the backend.'), 'command_id' => $cmd_id);
    }

    protected function get_commands($command_id = 0)
    {
        $args = $this->request;
        $data = array();
        $opts = array();

        if (!empty($command_id)) {
            $opts['command_id'] = $command_id;
        }

        // Do some filtering
        if (array_key_exists('status_code', $args)) {
            $opts['status_code'] = $args['status_code'];
        }

        $arr = get_command_status_xml($opts, true, false);
        if (!empty($arr)) {
            foreach ($arr as $i => $cmd) {
                unset($cmd['group_id']);
                unset($cmd['frequency_type']);
                unset($cmd['frequency_units']);
                unset($cmd['frequency_interval']);
                unset($cmd['beneficiary_id']);
                unset($cmd['command_data']);
                $data[] = $cmd;
            }
        }

        return $data;
    }

    protected function get_users($user_id = 0)
    {
        $data = array();
        $opts = array();

        if (!empty($user_id)) {
            $opts = array("user_id" => $user_id);
        }

        // Get users
        $users = get_users($opts);

        // Add in the data
        $data['records'] = count($users);
        foreach ($users as $u) {
            $data['users'][] = array('user_id' => $u['user_id'],
                                     'username' => $u['username'],
                                     'name' => $u['name'],
                                     'email' => $u['email'],
                                     'enabled' => $u['enabled']);
        }

        return $data;
    }

    /**
     * Authenticate the user and return an auth token that can be
     * used to log into the web interface.
     */
    protected function authenticate_user()
    {
        $args = $this->request;

        $username = strtolower(grab_array_var($args, "username", ""));
        $password = grab_array_var($args, "password", "");
        $valid_min = floatval(grab_array_var($args, "valid_min", 5));

        // Verify that everything we need is here
        if (empty($username) || empty($password)) {
            return array("error" => _("Must be valid username and password."));
        }

        // Verify user credentials (local, AD, LDAP, etc)
        $msg = array();
        if (check_login_credentials($username, $password, $msg)) {
            $user_id = get_user_id($username);
            $valid_until = time() + ($valid_min * 60);
            $auth_token = user_generate_auth_token($user_id, $valid_until);
        } else {
            $message = "";
            if (!empty($msg)) {
                $message = $msg[0]; // Only display first message
            }
            $data = array('username' => $username, 'message' => $message, 'error' => 1);
            return $data;
        }

        // Generate auth token and return it
        $data = array('username' => $username,
                      'user_id' => $user_id,
                      'auth_token' => $auth_token,
                      'valid_min' => $valid_min,
                      'valid_until' => date('r', $valid_until));

        return $data;
    }

    protected function add_user()
    {
        $missing = array();
        $args = $this->request;

        if (in_demo_mode()) {
            return array("error" => _("Can not use this action in demo mode."));
        }

        // Get values
        $username = strtolower(grab_array_var($args, "username", ""));
        $password = grab_array_var($args, "password", "");
        $email = grab_array_var($args, "email", "");
        $name = grab_array_var($args, "name", "");
        $level = grab_array_var($args, "auth_level", "user");
        $forcechangepass = grab_array_var($args, "force_pw_change", 1);
        $email_info = grab_array_var($args, "email_info", 1);
        $add_contact = grab_array_var($args, "monitoring_contact", 1);
        $enable_notifications = grab_array_var($args, "enable_notifications", 1);
        $language = grab_array_var($args, "language", "");
        $date_format = grab_array_var($args, "defaultDateFormat", DF_ISO8601);
        $number_format = grab_array_var($args, "defaultNumberFormat", NF_2);
        $authorized_for_all_objects = grab_array_var($args, "can_see_all_hs", 0);
        $authorized_for_all_object_commands = grab_array_var($args, "can_control_all_hs", 0);
        $authorized_to_configure_objects = grab_array_var($args, "can_reconfigure_hs", 0);
        $authorized_for_monitoring_system = grab_array_var($args, "can_control_engine", 0);
        $advanced_user = grab_array_var($args, "can_use_advanced", 0);
        $readonly_user = grab_array_var($args, "read_only", 0);
        $api_enabled = grab_array_var($args, "api_enabled", 0);

        // Auth type variables
        $auth_type = grab_array_var($args, "auth_type", "local");
        $allow_local = grab_array_var($args, "allow_local", 0);
        $auth_server_id = grab_array_var($args, "auth_server_id", "");
        $ad_username = grab_array_var($args, "ad_username", "");
        $ldap_dn = grab_array_var($args, "ldap_dn", "");

        // Verify that everything we need is here
        $required = array('username', 'email', 'name');
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        // Verify auth level
        $auth = '';
        if ($level == "user") {
            $level = 1;
        } else if ($level == "admin") {
            $level = 255;
        } else {
            $auth = array('auth_level' => _('Must be either user or admin.'));
        }

        // Verify auth types
        if ($auth_type == 'local') {
            if (empty($password)) {
                $missing[] = "password";
            }
        } else {
            if (empty($auth_server_id)) {
                $missing[] = "auth_server_id";
            } else {
                $forcechangepass = 0;
                $email_info = 0;

                // Verify that the auth server provided exists
                if (auth_server_get($auth_server_id) === false) {
                    $auth = array('auth_server_id' => _('Not a valid auth_server_id given. Auth server IDs can be found using GET system/authserver.'));
                }

                // Verify password exists if local auth allowed
                if ($allow_local && empty($password)) {
                    $auth = array('password' => _('You must enter a password if you are allowing local authentication if the AD/LDAP server cannot be reached.'));
                } else if (empty($password)) {
                    $password = random_string(16);
                }

            }
        }

        // We are missing required fields
        if (count($missing) > 0 || !empty($auth)) {
            $errormsg = array('error' => _('Could not create user. Missing required fields.'));
            if (!empty($auth)) { $errormsg['messages'] = $auth; }
            if (count($missing) > 0) { $errormsg['missing'] = $missing; }
            return $errormsg;
        }

        // Verify that user doesn't exist
        if (get_user_id($username) !== null) {
            return array("error" => _('A user with the username provided already exists.'));
        }

        // Verify that password is proper length
        if (strlen($password) > 72) {
            return array("error" => _('Passwords must be less than 72 characters long.'));
        }

        // If everything looks okay then proceed to create the user
        $user_id = add_user_account($username, $password, $name, $email, $level, $forcechangepass, $add_contact, $api_enabled, $errmsg);

        set_user_meta($user_id, 'name', $name);
        set_user_meta($user_id, 'language', $language);
        set_user_meta($user_id, 'theme', $theme);
        set_user_meta($user_id, "date_format", $date_format);
        set_user_meta($user_id, "number_format", $number_format);
        set_user_meta($user_id, "authorized_for_all_objects", $authorized_for_all_objects);
        set_user_meta($user_id, "authorized_for_all_object_commands", $authorized_for_all_object_commands);
        set_user_meta($user_id, "authorized_to_configure_objects", $authorized_to_configure_objects);
        set_user_meta($user_id, "authorized_for_monitoring_system", $authorized_for_monitoring_system);
        set_user_meta($user_id, "advanced_user", $advanced_user);
        set_user_meta($user_id, "readonly_user", $readonly_user);

        // Set authentication settings
        set_user_meta($user_id, "auth_type", $auth_type);
        set_user_meta($user_id, "allow_local", $allow_local);
        if ($auth_type == 'ad') {
            set_user_meta($user_id, "auth_server_id", $auth_server_id);
            set_user_meta($user_id, "ldap_ad_username", $ad_username);
        } else if ($auth_type == 'ldap') {
            set_user_meta($user_id, "auth_server_id", $auth_server_id);
            set_user_meta($user_id, "ldap_ad_dn", $ldap_dn);
        }

        if ($add_contact) {
            set_user_meta($user_id, "enable_notifications", $enable_notifications);
        }

        // Update nagios cgi config file
        update_nagioscore_cgi_config();

        if ($email_info) {
            $url = get_option("url");

            // Use this for debug output in PHPmailer log
            $debugmsg = "";

            // Set where email is coming from for PHPmailer log
            $send_mail_referer = "api/includes/utils-api.inc.php > API - Add User";

            $msg = _("An account has been created for you to access Nagios XI.  You can login using the following information:\n\nUsername: %s\nPassword: %s\nURL: %s\n\n");
            $message = sprintf($msg, $username, $password, $url);
            $opts = array(
                "to" => $email,
                "subject" => _("Nagios XI Account Created"),
                "message" => $message,
            );
            send_email($opts, $debugmsg, $send_mail_referer);
        }

        // Log it
        if ($level == L_GLOBALADMIN) {
            send_to_audit_log("User account '" . $username . "' was created with GLOBAL ADMIN privileges", AUDITLOGTYPE_SECURITY);
        }

        $ret = array('success' => _('User account') . ' ' . $username . ' ' . _('was added successfully!'));
        $ret['user_id'] = $user_id;
        return $ret;
    }

    protected function delete_user($user_id = 0)
    {
        if (in_demo_mode()) {
            return array("error" => _("Can not use this action in demo mode."));
        }

        if (!is_valid_user_id($user_id)) {
            return array("error" => _('User with') . ' ID ' . $user_id . ' ' . _('does not exist.'));
        }

        if ($user_id == $_SESSION["user_id"]) {
            return array("error" => _('Cannot delete the user you are currently using.'));
        }

        // Remove user from http
        update_nagioscore_cgi_config();
        $args = array(
            "username" => get_user_attr($user_id, 'username'),
        );
        submit_command(COMMAND_NAGIOSXI_DEL_HTACCESS, serialize($args));
        delete_user_id($user_id);

        return _('Success. User removed.');
    }

    protected function get_system_status()
    {
        return get_program_status_xml_output(array(), true);
    }

    protected function get_system_info()
    {
        $ret = array(
            'product' => get_product_name(true),
            'version' => get_product_version(),
            'version_major' => get_product_version('major'),
            'version_minor' => get_product_version('minor'),
            'build_id' => get_build_id(),
            'release' => get_product_release()
        );
        return $ret;
    }

    // =========================
    // Configuration
    // =========================

    protected function create_cfg_host()
    {
        global $firsthost;
        global $myImportClass;
        $args = $this->request;
        $applyconfig = false;

        // Verify required arguments were sent
        $required = array('host_name', 'address', 'max_check_attempts', 'check_period', 'notification_interval', 'notification_period');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (!array_key_exists('contacts', $args) && !array_key_exists('contact_groups', $args)) {
            $missing[] = 'contacts OR contact_groups';
        }
        $required[] = 'contacts';
        $required[] = 'contact_groups';

        if (count($missing) > 0 && empty($args['force'])) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_HOST);
        foreach ($required as $r) {
            if (array_key_exists($r, $args)) {
                $obj[$r] = $args[$r];
            }
        }

        // Add any optional components that we need to
        $others = array('alias', 'display_name', 'parents', 'hourly_value', 'hostgroups', 'check_command', 'use', 'initial_state', 'check_interval', 'retry_interval', 'active_checks_enabled', 'passive_checks_enabled', 'obsess_over_host', 'check_freshness', 'freshness_threshold', 'event_handler', 'event_handler_enabled', 'low_flap_threshold', 'high_flap_threshold', 'flap_detection_enabled', 'flap_detection_options', 'process_perf_data', 'retain_status_information', 'retain_nonstatus_information', 'first_notification_delay', 'notification_options', 'notifications_enabled', 'stalking_options', 'notes', 'notes_url', 'action_url', 'icon_image', 'icon_image_alt', 'vrml_image', 'statusmap_image', '2d_coords', '3d_coords', 'name', 'register');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                if ($o == 'check_command') { $args[$o] = stripslashes($args[$o]); }
                $obj[$o] = $args[$o];
            }
        }

        // Add any free variables that we find (they must start with an underscore)
        foreach ($args as $key => $value) {
            if (substr($key, 0, 1) === '_') {
                $obj[$key] = $value;
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        $str = get_cfg_objects_str($objs, $firsthost);
        $myImportClass->fileImport($str, 1, true);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['host_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_service()
    {
        global $firsthost;
        global $myImportClass;
        $args = $this->request;
        $applyconfig = false;

        // Verify required arguments were sent
        $required = array('host_name', 'service_description', 'max_check_attempts', 'check_interval', 'retry_interval', 'check_period', 'notification_interval', 'notification_period');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (!array_key_exists('contacts', $args) && !array_key_exists('contact_groups', $args)) {
            $missing[] = 'contacts OR contact_groups';
        }
        $required[] = 'contacts';
        $required[] = 'contact_groups';

        if (count($missing) > 0 && empty($args['force'])) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_SERVICE);
        foreach ($required as $r) {
            if (array_key_exists($r, $args)) {
                $obj[$r] = $args[$r];
            }
        }

        // Add any optional components that we need to
        $others = array('hostgroup_name', 'display_name', 'parents', 'hourly_value', 'servicegroups', 'is_volatile', 'use', 'initial_state', 'active_checks_enabled', 'passive_checks_enabled', 'obsess_over_service', 'check_freshness', 'check_command', 'freshness_threshold', 'event_handler', 'event_handler_enabled', 'low_flap_threshold', 'high_flap_threshold', 'flap_detection_enabled', 'flap_detection_options', 'process_perf_data', 'retain_status_information', 'retain_nonstatus_information', 'first_notification_delay', 'notification_options', 'notifications_enabled', 'stalking_options', 'notes', 'notes_url', 'action_url', 'icon_image', 'icon_image_alt', 'name', 'register', 'config_name');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                if ($o == 'check_command') { $args[$o] = stripslashes($args[$o]); }
                $obj[$o] = $args[$o];
            }
        }

        // Add any free variables that we find (they must start with an underscore)
        foreach ($args as $key => $value) {
            if (substr($key, 0, 1) === '_') {
                $obj[$key] = $value;
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        $str = get_cfg_objects_str($objs, $firsthost);
        $myImportClass->fileImport($str, 1, true);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['host_name'] . ' :: ' . $args['service_description'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_hostgroup()
    {
        global $firsthost;
        global $myImportClass;
        $args = $this->request;
        $applyconfig = false;

        $required = array('hostgroup_name', 'alias');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_HOSTGROUP);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        $others = array('members', 'hostgroup_members', 'notes', 'notes_url', 'action_url');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        $str = get_cfg_objects_str($objs, $firsthost);
        $myImportClass->fileImport($str, 1, true);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['hostgroup_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_servicegroup()
    {
        global $firsthost;
        global $myImportClass;
        $args = $this->request;
        $applyconfig = false;

        $required = array('servicegroup_name', 'alias');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_SERVICEGROUP);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        $others = array('members', 'servicegroup_members', 'notes', 'notes_url', 'action_url');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        $str = get_cfg_objects_str($objs, $firsthost);
        $myImportClass->fileImport($str, 1, true);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['servicegroup_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_command()
    {
        global $firsthost;
        global $myImportClass;
        $args = $this->request;
        $applyconfig = false;

        $required = array('command_name', 'command_line');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_COMMAND);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        // Commit the data to a file
        $objs = array($obj);
        $str = get_cfg_objects_str($objs, $firsthost);
        $myImportClass->fileImport($str, 1, true);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['command_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function remove_cfg_command()
    {
        $applyconfig = false;
        $missing = array();
        
        if (!array_key_exists('command_name', $this->request)) { $missing[] = 'command_name'; }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        # Import delete object function
        include_once('../../includes/components/ccm/includes/delete_object.inc.php');

        // Get Command ID
        $command_name = grab_array_var($this->request, 'command_name', '');
        $obj_id = nagiosccm_get_command_id($command_name);

        if ($obj_id <= 0) {
            return array('error' => _('Could not find a unique id for this command.'));
        }

        // Delete object
        $x = delete_object('command', $obj_id);

        // Error so display information about error
        if ($x[0] != 0) {
            return array('error' => _('Command cannot be deleted using this method. Must be deleted through the CCM.'),
                         'details' => $x[1]);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $this->request)) {
            if ($this->request['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Removed') . ' ' . $command_name . ' ' . _('from the system.') . $msg);
    }

    protected function remove_cfg_host()
    {
        $applyconfig = false;
        $missing = array();
        
        if (!array_key_exists('host_name', $this->request)) { $missing[] = 'host_name'; }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        $host = grab_array_var($this->request, 'host_name', '');
        $hid = nagiosql_get_host_id($host);

        if ($hid <= 0) {
            return array('error' => _('Could not find a unique id for this host.'));
        }

        if (nagiosccm_can_host_be_deleted($host) == false) {
            return array('error' => _('Host cannot be deleted using this method. Must be deleted through the CCM.'));
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $this->request)) {
            if ($this->request['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        delete_nagioscore_host($host, $applyconfig);
        return array('success' => _('Removed') . ' ' . $host . ' ' . _('from the system.') . $msg);
    }

    protected function remove_cfg_service()
    {
        $applyconfig = false;
        $missing = array();

        if (!array_key_exists('host_name', $this->request)) { $missing[] = 'host_name'; }
        if (!array_key_exists('service_description', $this->request)) { $missing[] = 'service_description'; }
        
        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        $host = grab_array_var($this->request, 'host_name', '');
        $service = grab_array_var($this->request, 'service_description', '');
        $sid = nagiosql_get_service_id($host, $service);

        if ($sid <= 0) {
            return array('error' => _('Could not find a unique id for this service.'));
        }

        if (can_service_be_deleted($host, $service) == false) {
            return array('error' => _('Service cannot be deleted using this method. Must be deleted through the CCM.'));
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $this->request)) {
            if ($this->request['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        delete_nagioscore_service($host, $service, $applyconfig);
        return array('success' => _('Removed') . ' ' . $host . ' :: ' . $service . ' ' . _('from the system.') . $msg);
    }

    // =========================================
    // Fusion Connections
    // =========================================

    protected function add_fuse_host()
    {
        global $ccmDB;
        $args = $this->request;
        $host = $ccmDB->escape_string($args['host']);

        if (empty($host)) {
            return array('error' => _('No host given.'));
        }

        // Add the fused host to the fused host list
        $fuse_hosts = explode(',', get_option('frame_options_allowed_fusion_hosts', ''));
        $add = true;
        foreach ($fuse_hosts as $k => $h) {
            if ($h == $host) {
                $add = false;
            }
        }
        if ($add) {
            $fuse_hosts[] = $host;
        }
        set_option('frame_options_allowed_fusion_hosts', implode(',', $fuse_hosts));

        return array('success' => _("Fusion host added successfully."));
    }

    protected function remove_fuse_host($host='')
    {
        if (empty($host)) {
            return array('error' => _('No host given.'));
        }

        // Remove fuse host from the list
        $fuse_hosts = explode(',', get_option('frame_options_allowed_fusion_hosts', ''));
        if ($fuse_hosts > 0) {
            foreach ($fuse_hosts as $k => $h) {
                if ($h == $host) {
                    unset($fuse_hosts[$k]);
                }
            }
            set_option('frame_options_allowed_fusion_hosts', implode(',', $fuse_hosts));
        } else {
            return array('error' => _("No fuse hosts exist."));
        }

        return array('success' => _("Fusion host removed successfully."));
    }

    // =========================================
    // Auth Servers
    // =========================================

    protected function get_auth_servers()
    {
        // Check if we are getting a single auth server by ID
        $server_id = grab_request_var('server_id', '');
        $data = array('records' => 0, 'authservers' => array());

        // Remove the auth server from the DB
        if (empty($server_id)) {
            $servers = auth_server_list();
            $data = array('records' => count($servers), 'authservers' => $servers);
        } else {
            $server = auth_server_get($server_id);
            if ($server) {
                $data = array('records' => 1, 'authservers' => array($server));
            }
        }

        return $data;
    }

    protected function add_auth_server()
    {
        $missing = array();
        $args = $this->request;
        $valid_methods = array('ldap', 'ad');

        $enabled = grab_request_var($args, 'enabled', 1);
        $conn_method = grab_array_var($args, 'conn_method', '');
        $base_dn = grab_array_var($args, 'base_dn', '');
        $domain_controllers = grab_array_var($args, 'ad_domain_controllers', '');
        $security_level = grab_array_var($args, 'security_level', 'none');
        $ldap_host = grab_array_var($args, 'ldap_host', '');
        $ldap_port = grab_array_var($args, 'ldap_port', '');
        $account_suffix = grab_array_var($args, 'ad_account_suffix', '');

        // Add default LDAP port if we need to
        if ($conn_method == 'ldap') {
            if (empty($ldap_port)) {
                $ldap_port = '389';
            }
        }

        // Verification checks
        if (!in_array($conn_method, $valid_methods)) {
            return array('error' => _('The conn_method you specified is not allowed. Choose ad or ldap only.'));
        }

        // Verify user has all items based on conn_method
        if ($conn_method == 'ad') {
            if (empty($domain_controllers)) {
                $missing[] = "ad_domain_controllers";
            }
            if (empty($account_suffix)) {
                $missing[] = "ad_account_suffix";
            }

            if (!empty($missing)) {
                return array('error' => _('If you are using AD you must pass the following options.'),
                             'missing' => $missing);
            }
        } else {
            if (empty($ldap_host)) {
                $missing[] = "ldap_host";
            }

            if (!empty($missing)) {
                return array('error' => _('If you are using LDAP you must pass the following option.'),
                             'missing' => $missing);
            }
        }

        // Create the auth server
        $server = array(
            "id" => uniqid(),
            "enabled" => $enabled,
            "conn_method" => $conn_method,
            "ad_account_suffix" => $account_suffix,
            "ad_domain_controllers" => $domain_controllers,
            "base_dn" => $base_dn,
            "security_level" => $security_level,
            "ldap_port" => $ldap_port,
            "ldap_host" => $ldap_host
        );
        $server_id = auth_server_add($server);

        return array('success' => 'Auth server created successfully.',
                     'server_id' => $server_id);
    }

    protected function remove_auth_server($id='')
    {
        if (empty($id)) {
            return array('error' => _('No auth server ID given.'));
        }

        // Remove the auth server
        $removed = auth_server_remove($id);

        // If the server wasn't removed, we should not save it and error out
        if (!$removed) {
            return array('error' => _("Could not remove auth server with ID given."));
        }

        return array('success' => _("Removed auth server successfully."));
    }

    // =========================================
    // Scheduled Downtime
    // =========================================

    protected function add_scheduled_downtime()
    {
        $errors = 0;
        $error_items = array();
        $scheduled = array();

        $author = get_user_attr($_SESSION['user_id'], 'username');
        $comment = grab_request_var('comment', '');
        $triggered_by = intval(grab_request_var('triggered_by', 0));
        $flexible = intval(grab_request_var('flexible', 0));
        $duration = intval(grab_request_var('duration', 0));
        $start = intval(grab_request_var('start', 0));
        $end = intval(grab_request_var('end', 0));

        // Objects
        $hosts = grab_request_var('hosts', array());
        $services = grab_request_var('services', array());
        $hostgroups = grab_request_var('hostgroups', array());
        $servicegroups = grab_request_var('servicegroups', array());

        // Host only
        $child_hosts = intval(grab_request_var('child_hosts', 0));
        $all_services = intval(grab_request_var('all_services', 0));

        // Hostgroups and Servicegroups only
        $only = grab_request_var('only', '');
        if ($only != 'services' && $only != 'hosts') {
            $only = '';
        }

        // Check for all empty
        if (empty($hosts) && empty($services) && empty($hostgroups) && empty($servicegroups)) {
            return array('error' => _("You must enter at least one hosts[], services[], hostgroups[], or servicegroups[]."));
        }

        // Comments are required
        if (empty($comment)) {
            return array('error' => _("You must enter a comment."));
        }

        // Check for valid timestamp
        if ($start > $end) {
            return array('error' => _("Your start time cannot be after your end time."));
        }

        // Generate command arguments
        $args = array("comment_data" => $comment,
                      "comment_author" => $author,
                      "trigger_id" => $triggered_by,
                      "start_time" => $start,
                      "end_time" => $end,
                      "fixed" => 1);

        // If flexible, add a duration value
        if ($flexible == 1) {
            $args['fixed'] = 0;
            $args['duration'] = $duration;
        }

        // Do any hosts we were sent
        if (!empty($hosts)) {
            foreach ($hosts as $host) {
                $host_args = $args;
                $host_args['host_name'] = $host;

                $cmd = NAGIOSCORE_CMD_SCHEDULE_HOST_DOWNTIME;
                if ($child_hosts == 2) {
                    $cmd = NAGIOSCORE_CMD_SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME;
                } else if ($child_hosts == 1) {
                    $cmd = NAGIOSCORE_CMD_SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME;
                }

                // Run command
                $core_cmd = core_get_raw_command($cmd, $host_args);
                $x = core_submit_command($core_cmd, $output);
                if (!$x) {
                    $errors++;
                    $error_items['hosts'][] = $host;
                }

                $scheduled['hosts'][] = $host;

                // If all services on host
                if ($all_services == 1) {
                    $cmd = NAGIOSCORE_CMD_SCHEDULE_HOST_SVC_DOWNTIME;
                    $core_cmd = core_get_raw_command($cmd, $host_args);
                    $x = core_submit_command($core_cmd, $output);
                    if (!$x) {
                        $errors++;
                        $error_items['services'][$host][] = _('ALL');
                    }
                    $scheduled['services'][$host][] = _('ALL');
                }
            }
        }

        // Do any services we were sent
        if (!empty($services)) {
            foreach ($services as $host => $service) {
                $service_args = $args;
                $service_args['host_name'] = $host;
                $service_args['service_name'] = $service;
                $core_cmd = core_get_raw_command(NAGIOSCORE_CMD_SCHEDULE_SVC_DOWNTIME, $service_args);
                $x = core_submit_command($core_cmd, $output);
                if (!$x) {
                    $errors++;
                    $error_items['services'][$host][] = $service;
                }
                $scheduled['services'][$host][] = $service;
            }
        }

        // Do host groups
        if (!empty($hostgroups)) {
            foreach ($hostgroups as $hostgroup) {
                $hostgroup_args = $args;
                $hostgroup_args['host_group'] = $hostgroup;
                $core_host_cmd = core_get_raw_command(NAGIOSCORE_CMD_SCHEDULE_HOSTGROUP_HOST_DOWNTIME, $hostgroup_args);
                $core_svc_cmd = core_get_raw_command(NAGIOSCORE_CMD_SCHEDULE_HOSTGROUP_SVC_DOWNTIME, $hostgroup_args);

                // Add hosts if we are not only adding services
                if ($only != 'services') {
                    $x = core_submit_command($core_host_cmd, $output);
                    if (!$x) {
                        $errors++;
                        $error_items['hostgroups'][$hostgroup][] = _('HOSTS');
                    }
                    $scheduled['hostgroups'][$hostgroup][] = _('HOSTS');
                }

                // Add services if we are not only adding hosts
                if ($only != 'hosts') {
                    $x = core_submit_command($core_host_cmd, $output);
                    if (!$x) {
                        $errors++;
                        $error_items['hostgroups'][$hostgroup][] = _('SERVICES');
                    }
                    $scheduled['hostgroups'][$hostgroup][] = _('SERVICES');
                }
            }
        }

        // Do service groups
        if (!empty($servicegroups)) {
            foreach ($servicegroups as $servicegroup) {
                $servicegroup_args = $args;
                $servicegroup_args['service_group'] = $servicegroup;
                $core_host_cmd = core_get_raw_command(NAGIOSCORE_CMD_SCHEDULE_SERVICEGROUP_HOST_DOWNTIME, $servicegroup_args);
                $core_svc_cmd = core_get_raw_command(NAGIOSCORE_CMD_SCHEDULE_SERVICEGROUP_SVC_DOWNTIME, $servicegroup_args);

                // Add hosts if we are not only adding services
                if ($only != 'services') {
                    $x = core_submit_command($core_host_cmd, $output);
                    if (!$x) {
                        $errors++;
                        $error_items['servicegroups'][$servicegroup][] = _('HOSTS');
                    }
                    $scheduled['servicegroups'][$servicegroup][] = _('HOSTS');
                }

                // Add services if we are not only adding hosts
                if ($only != 'hosts') {
                    $x = core_submit_command($core_host_cmd, $output);
                    if (!$x) {
                        $errors++;
                        $error_items['servicegroups'][$servicegroup][] = _('SERVICES');
                    }
                    $scheduled['servicegroups'][$servicegroup][] = _('SERVICES');
                }
            }
        }

        // Return data on what was created
        if ($errors) {
            return array(
                'error' => _("Errors sending schedule downtime command(s) to Core."),
                'scheduled' => $scheduled,
                'not_scheduled' => $error_items
            );
        }

        return array(
            'success' => _("Schedule downtime command(s) sent successfully."),
            'scheduled' => $scheduled
        );
    }

    protected function remove_scheduled_downtime($id='')
    {
        global $db_tables;
        $errors = 0;

        if (empty($id)) {
            return array('error' => _("You must specify a scheduled downtime ID."));
        }

        // If passed a list, do all of them
        $downtime_ids = array();
        if (strpos($id, ',') === false) {
            $downtime_ids[] = intval($id);
        } else {
            $ids = explode(',', $id);
            foreach ($ids as $i) {
                $downtime_ids[] = intval(trim($i));
            }
        }

        // Try to remove the scheduled downtime
        foreach ($downtime_ids as $downtime_id) {
            $sql = "SELECT downtime_type FROM " . $db_tables[DB_NDOUTILS]['scheduleddowntime'] . " WHERE internal_downtime_id = '" . $downtime_id . "'";
            $rs = exec_sql_query(DB_NDOUTILS, $sql);
            $dt = $rs->GetArray();

            // Could not find the downtime in the NDO db
            if (empty($dt)) {
                continue;
            }

            // Delete downtime based on type
            if ($dt[0]['downtime_type'] == 2) {
                $cmd = NAGIOSCORE_CMD_DEL_HOST_DOWNTIME;
            } else {
                $cmd = NAGIOSCORE_CMD_DEL_SVC_DOWNTIME;
            }

            $core_cmd = core_get_raw_command($cmd, array('downtime_id' => $downtime_id));
            $x = core_submit_command($core_cmd, $output);
            if (!$x) { $errors++; }
        }

        // Check for errors
        if ($errors) {
            return array('error' => _("Errors sending remove downtime command(s) to Core."));
        }

        return array('success' => _("Remove downtime command(s) sent successfully."));
    }

    // =========================================
    // Raw Config Import
    // =========================================

    protected function raw_cfg_import()
    {
        global $myImportClass;
        $applyconfig = false;
        $overwrite = intval(grab_request_var('overwrite', 0));

        $import = file_get_contents("php://input");
        $x = $myImportClass->fileImport($import, $overwrite, true);

        // Catch import error
        if ($x == 1) {
            return array('error' => _('Import failed. Configuration passed may not be formatted properly.'));
        }

        // Check if we should applyconfig nagios after
        if (grab_request_var('applyconfig', 0)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Imported configuration data.') . $msg);
    }

    // =========================================
    // Run Core Command
    // =========================================

    protected function core_command()
    {
        global $nagioscore_cmds;
        $cmd = grab_request_var('cmd', '');

        // Validate command is available
        if (empty($cmd)) {
            return array('error' => _("You must specify an external command."));
        }

        if (strpos($cmd, ';') !== false) {
            $cmd_parts = explode(';', $cmd);
            $cmd_name = $cmd_parts[0];
        } else {
            $cmd_name = $cmd;
        }

        // If no command exists, exit out
        $valid_cmds = array_values($nagioscore_cmds);
        if (!in_array($cmd_name, $valid_cmds)) {
            return array('error' => _("You must specify a valid external command."));
        }

        // Replace macros
        $username = get_user_attr($_SESSION['user_id'], 'username');
        $cmd = str_replace("%user%", $username, $cmd);

        // If everything is good, send the full command to the nagios.cmd file
        // TODO: We can improve this and not have to write out the command manually
        $output = "";
        $success = core_submit_command($cmd, $output);

        if (!$success) {
            return array('error' => $output);
        }

        return array('cmd' => $cmd, 'success' => $output);
    }

}

function get_rrd_json_output($args) {

    if (in_demo_mode()) {
        return array("error" => _("Can not use this action in demo mode."));
    }

    if (empty($args['host_name'])) {
        return array('error' => _('You must provide host_name.'));
    }

    include_once('../../includes/utils-rrdexport.inc.php');

    // get values and defaults
    $host = $args['host_name'];
    $service = (empty($args['service_description']) ? "_HOST_" : $args['service_description']);
    $start = "";
    $end = "";
    $step = "";
    $columns_to_display = array();

    // check for proper timestamps - if *you* want the data, *you* convert to timestamp
    if (!empty($args['start'])) {
        $start = date("U", $args['start']);
        if (!$start)
            return array("error" => _("Start time must be in Unix timestamp format."));
    }
    if (!empty($args['end'])) {
        $end = date("U", $args['end']);
        if (!$end)
            return array("error" => _("End time must be in Unix timestamp format."));
    }

    // check for proper step value, default to 300 if nothing or invalid value
    if (!empty($args['step']) && is_numeric($args['step']))
        $step = $args['step'];

    // check for columns specified - if found, attempt to build an array of acceptable ones
    if (!empty($args['columns'])) {
        if (is_array($args['columns'])) {
            foreach($args['columns'] as $column) {
                $columns_to_display[] = $column;
            }
        } else {
            $columns_to_display[] = $args['columns'];
        }
    }
    
    $output = get_rrd_data($host, $service, 'json', $start, $end, $step, $columns_to_display);

    if (!$output) {
        return array("error" => _("No valid data returned."));
    } else {
        return $output;
    }
}

function get_bpi_data($type='json')
{
    define('CLI', true);

    global $bpi_config;
    global $bpi_errors;
    global $bpi_objects;
    global $bpi_errors;
    global $bpi_obj_count;
    global $bpi_state_count;
    global $bpi_unique;
    global $host_details;

    $bpi_config = true;
    $bpi_errors = '';
    $bpi_objects = array();
    $bpi_errors = '';
    $bpi_obj_count = 0;
    $bpi_state_count = 0;
    $bpi_unique = 0;
    $host_details = array();

    require_once('../../includes/components/nagiosbpi/inc.inc.php');

    $data = bpi_xml_fetch();

    if ($type == 'json') {
        $json_arr = array();
        $data = simplexml_load_string($data);
        foreach ($data->group as $group) {
            $json_arr[strval($group['id'])] = array(
                'name' => strval($group->name),
                'title' => strval($group->title),
                'type' => strval($group->type),
                'status_text' => strval($group->status_text),
                'current_state' => intval($group->current_state),
                'problems' => intval($group->problems),
                'health' => floatval($group->health),
                'warning' => floatval($group->warning),
                'critical' => floatval($group->critical)
            );
        }
        $data = $json_arr;
    }

    return $data;
}